<?php

class Paddle_Subscription{
    /**
     * Get subscription data.
     * @param WC_Order $order
     * @return array
     */
    public function get_data(WC_Order $order): array
    {
        if (empty($order)) {
            wc_add_notice(__('Subscriptions data not found.', 'wp-paymenttory'), 'error');
        }

        $order_items = $order->get_items();
        $order_item = reset($order_items);
        $product = $order_item->get_product();

        $plan_name = $product->get_name();

        $plan_length = WC_Subscriptions_Product::get_interval($product);
        $plan_type = WC_Subscriptions_Product::get_period($product);

        $trial_length = WC_Subscriptions_Product::get_trial_length($product);
        $trial_type = WC_Subscriptions_Product::get_trial_period($product);

        $recurring_amount = WC_Subscriptions_Order::get_recurring_total($order);

        $trial_period_days = 0;
        switch (strtolower($trial_type)) {
            case 'week':
                $trial_period_days = 7;
                break;

            case 'month':
                $trial_period_days = 30;
                break;

            case 'year':
                $trial_period_days = 365;
                break;

            default:
            case 'day':
                $trial_period_days = 1;
                break;
        }
        $trial_days = $trial_period_days * $trial_length;

        $paddle_plan_name = strtoupper(sprintf(
            '%s [%s_%s_%s:T_%s]',
            $plan_name,
            $plan_length,
            $plan_type,
            $recurring_amount,
            $trial_days
        ));

        return array(
            'plan_name'         => $plan_name,
            'paddle_plan_name'  => $paddle_plan_name,
            'plan_type'         => $plan_type,
            'plan_length'       => $plan_length,
            'trial_days'        => $trial_days,
            'initial_amount'    => $order->get_total(),
            'recurring_amount'  => $recurring_amount,
        );
    }

    /**
     * @param $order
     * @param $settings
     * @return int
     * @throws \Exception
     */
    public function get_paddle_subs_plan($order, Paymenttory_Settings $settings)
    {

        $subscription_data = $this->get_data($order);

        $api = new Paddle\API(
            $settings->get('paddle_vendor_id'),
            $settings->get('paddle_api_key'),
            45,
            $settings->is_sandbox()
        );

        // list of paddle subscription plans
        $subscriptions = $api->subscription()->listPlans();

        // Filter plan name where match $subscription_data['paddle_plan_name']
        $plans = array_filter($subscriptions, function ($subscription) use ($subscription_data) {
            return $subscription['name'] === $subscription_data['paddle_plan_name'];
        });

        // If plan not found then create new plan
        if (!count($plans)) {
            $planData = [
                'plan_name' => $subscription_data['paddle_plan_name'],
                'trial_days' => $subscription_data['trial_days'],
                'plan_length' => $subscription_data['plan_length'] ?? 1,
                'plan_type' => $subscription_data['plan_type'] ?? 'year',
                'initial_price_' . strtolower($order->get_currency()) => $subscription_data['initial_amount'],
                'recurring_price_' . strtolower($order->get_currency()) => $subscription_data['recurring_amount'],
                'currency' => get_woocommerce_currency(),
                'paddle_plan_name' => $subscription_data['paddle_plan_name']
            ];

            $response = $api->subscription()->createPlan($planData);

            $product_id = $response['product_id'];

        }else{
            $paddle_plan = reset($plans);
           $product_id = (int) $paddle_plan['id'];
        }
        return $product_id;

    }

}


